@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -75,6 +77,9 @@ class Cube : public BaseCube< eT, Cube<eT> >
   template<typename fill_type> inline Cube(const uword in_rows, const uword in_cols, const uword in_slices, const fill::fill_class<fill_type>& f);
   template<typename fill_type> inline Cube(const SizeCube& s,                                               const fill::fill_class<fill_type>& f);
   
+  inline Cube(const uword in_rows, const uword in_cols, const uword in_slices, const fill::scalar_holder<eT> f);
+  inline Cube(const SizeCube& s,                                               const fill::scalar_holder<eT> f);
+  
   inline            Cube(Cube&& m);
   inline Cube& operator=(Cube&& m);
   
@@ -404,11 +409,11 @@ class Cube : public BaseCube< eT, Cube<eT> >
   inline bool  empty() const;
   inline uword size()  const;
   
-  inline       eT& front();
-  inline const eT& front() const;
+  inline arma_warn_unused       eT& front();
+  inline arma_warn_unused const eT& front() const;
   
-  inline       eT& back();
-  inline const eT& back() const;
+  inline arma_warn_unused       eT& back();
+  inline arma_warn_unused const eT& back() const;
   
   inline void swap(Cube& B);
   
@@ -465,6 +470,7 @@ class Cube<eT>::fixed : public Cube<eT>
   inline fixed();
   inline fixed(const fixed<fixed_n_rows, fixed_n_cols, fixed_n_slices>& X);
   
+                                     inline fixed(const fill::scalar_holder<eT> f);
   template<typename fill_type>       inline fixed(const fill::fill_class<fill_type>& f);
   template<typename T1>              inline fixed(const BaseCube<eT,T1>& A);
   template<typename T1, typename T2> inline fixed(const BaseCube<pod_type,T1>& A, const BaseCube<pod_type,T2>& B);
