@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -177,11 +179,12 @@
 
 
 
-#if (defined(ARMA_USE_OPENMP) && defined(ARMA_USE_CXX11))
+#if defined(ARMA_USE_OPENMP)
   
   #define arma_applier_1_mp(operatorA, operatorB) \
     {\
-    _Pragma("omp parallel for schedule(static)")\
+    const int n_threads = mp_thread_limit::get();\
+    _Pragma("omp parallel for schedule(static) num_threads(n_threads)")\
     for(uword i=0; i<n_elem; ++i)\
       {\
       out_mem[i] operatorA P1[i] operatorB P2[i];\
@@ -190,9 +193,10 @@
   
   #define arma_applier_2_mp(operatorA, operatorB) \
     {\
+    const int n_threads = mp_thread_limit::get();\
     if(n_cols == 1)\
       {\
-      _Pragma("omp parallel for schedule(static)")\
+      _Pragma("omp parallel for schedule(static) num_threads(n_threads)")\
       for(uword count=0; count < n_rows; ++count)\
         {\
         out_mem[count] operatorA P1.at(count,0) operatorB P2.at(count,0);\
@@ -201,7 +205,7 @@
     else\
     if(n_rows == 1)\
       {\
-      _Pragma("omp parallel for schedule(static)")\
+      _Pragma("omp parallel for schedule(static) num_threads(n_threads)")\
       for(uword count=0; count < n_cols; ++count)\
         {\
         out_mem[count] operatorA P1.at(0,count) operatorB P2.at(0,count);\
@@ -209,7 +213,7 @@
       }\
     else\
       {\
-      _Pragma("omp parallel for schedule(static)")\
+      _Pragma("omp parallel for schedule(static) num_threads(n_threads)")\
       for(uword col=0; col<n_cols; ++col)\
         {\
         for(uword row=0; row<n_rows; ++row)\
@@ -222,7 +226,8 @@
   
   #define arma_applier_3_mp(operatorA, operatorB) \
     {\
-    _Pragma("omp parallel for schedule(static)")\
+    const int n_threads = mp_thread_limit::get();\
+    _Pragma("omp parallel for schedule(static) num_threads(n_threads)")\
     for(uword slice=0; slice<n_slices; ++slice)\
       {\
       for(uword col=0; col<n_cols; ++col)\
@@ -260,7 +265,7 @@ eglue_core<eglue_type>::apply(outT& out, const eGlue<T1, T2, eglue_type>& x)
   typedef typename T1::elem_type eT;
   
   const bool use_at = (Proxy<T1>::use_at || Proxy<T2>::use_at);
-  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::openmp);
   
   // NOTE: we're assuming that the matrix has already been set to the correct size and there is no aliasing;
   // size setting and alias checking is done by either the Mat contructor or operator=()
@@ -272,7 +277,7 @@ eglue_core<eglue_type>::apply(outT& out, const eGlue<T1, T2, eglue_type>& x)
     {
     const uword n_elem = x.get_n_elem();
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(n_elem))
       {
       typename Proxy<T1>::ea_type P1 = x.P1.get_ea();
       typename Proxy<T2>::ea_type P2 = x.P2.get_ea();
@@ -329,7 +334,7 @@ eglue_core<eglue_type>::apply(outT& out, const eGlue<T1, T2, eglue_type>& x)
     const Proxy<T1>& P1 = x.P1;
     const Proxy<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_2_mp(=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_2_mp(=, -); }
@@ -367,13 +372,13 @@ eglue_core<eglue_type>::apply_inplace_plus(Mat<typename T1::elem_type>& out, con
   eT* out_mem = out.memptr();
   
   const bool use_at = (Proxy<T1>::use_at || Proxy<T2>::use_at);
-  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::openmp);
   
   if(use_at == false)
     {
     const uword n_elem = x.get_n_elem();
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(n_elem))
       {
       typename Proxy<T1>::ea_type P1 = x.P1.get_ea();
       typename Proxy<T2>::ea_type P2 = x.P2.get_ea();
@@ -427,7 +432,7 @@ eglue_core<eglue_type>::apply_inplace_plus(Mat<typename T1::elem_type>& out, con
     const Proxy<T1>& P1 = x.P1;
     const Proxy<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_2_mp(+=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_2_mp(+=, -); }
@@ -465,13 +470,13 @@ eglue_core<eglue_type>::apply_inplace_minus(Mat<typename T1::elem_type>& out, co
   eT* out_mem = out.memptr();
   
   const bool use_at = (Proxy<T1>::use_at || Proxy<T2>::use_at);
-  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::openmp);
   
   if(use_at == false)
     {
     const uword n_elem = x.get_n_elem();
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(n_elem))
       {
       typename Proxy<T1>::ea_type P1 = x.P1.get_ea();
       typename Proxy<T2>::ea_type P2 = x.P2.get_ea();
@@ -525,7 +530,7 @@ eglue_core<eglue_type>::apply_inplace_minus(Mat<typename T1::elem_type>& out, co
     const Proxy<T1>& P1 = x.P1;
     const Proxy<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_2_mp(-=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_2_mp(-=, -); }
@@ -563,13 +568,13 @@ eglue_core<eglue_type>::apply_inplace_schur(Mat<typename T1::elem_type>& out, co
   eT* out_mem = out.memptr();
   
   const bool use_at = (Proxy<T1>::use_at || Proxy<T2>::use_at);
-  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::openmp);
   
   if(use_at == false)
     {
     const uword n_elem = x.get_n_elem();
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(n_elem))
       {
       typename Proxy<T1>::ea_type P1 = x.P1.get_ea();
       typename Proxy<T2>::ea_type P2 = x.P2.get_ea();
@@ -623,7 +628,7 @@ eglue_core<eglue_type>::apply_inplace_schur(Mat<typename T1::elem_type>& out, co
     const Proxy<T1>& P1 = x.P1;
     const Proxy<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_2_mp(*=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_2_mp(*=, -); }
@@ -661,13 +666,13 @@ eglue_core<eglue_type>::apply_inplace_div(Mat<typename T1::elem_type>& out, cons
   eT* out_mem = out.memptr();
   
   const bool use_at = (Proxy<T1>::use_at || Proxy<T2>::use_at);
-  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (Proxy<T1>::use_mp || Proxy<T2>::use_mp) && (arma_config::openmp);
   
   if(use_at == false)
     {
     const uword n_elem = x.get_n_elem();
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(n_elem))
       {
       typename Proxy<T1>::ea_type P1 = x.P1.get_ea();
       typename Proxy<T2>::ea_type P2 = x.P2.get_ea();
@@ -721,7 +726,7 @@ eglue_core<eglue_type>::apply_inplace_div(Mat<typename T1::elem_type>& out, cons
     const Proxy<T1>& P1 = x.P1;
     const Proxy<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (Proxy<T1>::use_mp && Proxy<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (Proxy<T1>::use_mp && Proxy<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_2_mp(/=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_2_mp(/=, -); }
@@ -757,7 +762,7 @@ eglue_core<eglue_type>::apply(Cube<typename T1::elem_type>& out, const eGlueCube
   typedef typename T1::elem_type eT;
   
   const bool use_at = (ProxyCube<T1>::use_at || ProxyCube<T2>::use_at);
-  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::openmp);
   
   // NOTE: we're assuming that the cube has already been set to the correct size and there is no aliasing;
   // size setting and alias checking is done by either the Cube contructor or operator=()
@@ -769,7 +774,7 @@ eglue_core<eglue_type>::apply(Cube<typename T1::elem_type>& out, const eGlueCube
     {
     const uword n_elem = out.n_elem;
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(n_elem))
       {
       typename ProxyCube<T1>::ea_type P1 = x.P1.get_ea();
       typename ProxyCube<T2>::ea_type P2 = x.P2.get_ea();
@@ -827,7 +832,7 @@ eglue_core<eglue_type>::apply(Cube<typename T1::elem_type>& out, const eGlueCube
     const ProxyCube<T1>& P1 = x.P1;
     const ProxyCube<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_3_mp(=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_3_mp(=, -); }
@@ -866,13 +871,13 @@ eglue_core<eglue_type>::apply_inplace_plus(Cube<typename T1::elem_type>& out, co
   eT* out_mem = out.memptr();
   
   const bool use_at = (ProxyCube<T1>::use_at || ProxyCube<T2>::use_at);
-  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::openmp);
   
   if(use_at == false)
     {
     const uword n_elem = out.n_elem;
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(n_elem))
       {
       typename ProxyCube<T1>::ea_type P1 = x.P1.get_ea();
       typename ProxyCube<T2>::ea_type P2 = x.P2.get_ea();
@@ -926,7 +931,7 @@ eglue_core<eglue_type>::apply_inplace_plus(Cube<typename T1::elem_type>& out, co
     const ProxyCube<T1>& P1 = x.P1;
     const ProxyCube<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_3_mp(+=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_3_mp(+=, -); }
@@ -965,13 +970,13 @@ eglue_core<eglue_type>::apply_inplace_minus(Cube<typename T1::elem_type>& out, c
   eT* out_mem = out.memptr();
   
   const bool use_at = (ProxyCube<T1>::use_at || ProxyCube<T2>::use_at);
-  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::openmp);
   
   if(use_at == false)
     {
     const uword n_elem = out.n_elem;
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(n_elem))
       {
       typename ProxyCube<T1>::ea_type P1 = x.P1.get_ea();
       typename ProxyCube<T2>::ea_type P2 = x.P2.get_ea();
@@ -1025,7 +1030,7 @@ eglue_core<eglue_type>::apply_inplace_minus(Cube<typename T1::elem_type>& out, c
     const ProxyCube<T1>& P1 = x.P1;
     const ProxyCube<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_3_mp(-=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_3_mp(-=, -); }
@@ -1064,13 +1069,13 @@ eglue_core<eglue_type>::apply_inplace_schur(Cube<typename T1::elem_type>& out, c
   eT* out_mem = out.memptr();
   
   const bool use_at = (ProxyCube<T1>::use_at || ProxyCube<T2>::use_at);
-  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::openmp);
   
   if(use_at == false)
     {
     const uword n_elem = out.n_elem;
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(n_elem))
       {
       typename ProxyCube<T1>::ea_type P1 = x.P1.get_ea();
       typename ProxyCube<T2>::ea_type P2 = x.P2.get_ea();
@@ -1124,7 +1129,7 @@ eglue_core<eglue_type>::apply_inplace_schur(Cube<typename T1::elem_type>& out, c
     const ProxyCube<T1>& P1 = x.P1;
     const ProxyCube<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_3_mp(*=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_3_mp(*=, -); }
@@ -1163,13 +1168,13 @@ eglue_core<eglue_type>::apply_inplace_div(Cube<typename T1::elem_type>& out, con
   eT* out_mem = out.memptr();
   
   const bool use_at = (ProxyCube<T1>::use_at || ProxyCube<T2>::use_at);
-  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::cxx11 && arma_config::openmp);
+  const bool use_mp = (ProxyCube<T1>::use_mp || ProxyCube<T2>::use_mp) && (arma_config::openmp);
   
   if(use_at == false)
     {
     const uword n_elem = out.n_elem;
     
-    if(use_mp && (n_elem >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(n_elem))
       {
       typename ProxyCube<T1>::ea_type P1 = x.P1.get_ea();
       typename ProxyCube<T2>::ea_type P2 = x.P2.get_ea();
@@ -1223,7 +1228,7 @@ eglue_core<eglue_type>::apply_inplace_div(Cube<typename T1::elem_type>& out, con
     const ProxyCube<T1>& P1 = x.P1;
     const ProxyCube<T2>& P2 = x.P2;
     
-    if(use_mp && (x.get_n_elem() >= ((is_cx<eT>::yes || (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)) ? (arma_config::mp_threshold/uword(2)) : (arma_config::mp_threshold))))
+    if(use_mp && mp_gate<eT, (ProxyCube<T1>::use_mp && ProxyCube<T2>::use_mp)>::eval(x.get_n_elem()))
       {
            if(is_same_type<eglue_type, eglue_plus >::yes) { arma_applier_3_mp(/=, +); }
       else if(is_same_type<eglue_type, eglue_minus>::yes) { arma_applier_3_mp(/=, -); }
